
local _,L = ...
local rematch = Rematch
local settings
local saved

local DEBUG = false
local function debugp(...) if DEBUG then print(...) end end

-- loadin is a team table filled with the pets and abilities to load
-- LoadTeam(teamName) fills the table
-- LoadLoadIn() actually loads the pets/abilities (and removes from table)
-- when loadin is empty, the team is done loading
local loadin = {{},{},{}}
local loadingTeamName -- name of team being loaded
local loadtimeout -- LoadLoadIn attempts (max 10)

function rematch:InitPetLoading()
	settings = RematchSettings
	saved = RematchSaved
end

-- loads the pets defined in loadin
-- nils the pets/abilities in loadin that loaded successfully
-- returns true if everything loaded
function rematch:LoadLoadIn()

	local loadout = rematch.info
	wipe(loadout)

	-- now attempt to load all pets/abilities that are defined
	for slot=1,3 do
		debugp("__ slot",slot,"__")
		loadout[1],loadout[2],loadout[3],loadout[4] = C_PetJournal.GetPetLoadOutInfo(slot)
		-- pet slot
		if loadin[slot][1] and loadin[slot][1] ~= loadout[1] then
			debugp("loading pet slot",slot,":",loadin[slot][1])
			if slot==1 then
				rematch:SummonedPetMayChange()
			end
			C_PetJournal.SetPetLoadOutInfo(slot,loadin[slot][1])
		end
		-- ability slots
		for i=1,3 do
			local abilityID = loadin[slot][i+1]
			if abilityID and loadout[i+1]~=abilityID then
				debugp("loading ability slot",i,":",abilityID)
				C_PetJournal.SetAbility(slot,i,abilityID)
			end
		end
	end

	-- now nil out the pets/abilities that loaded successfully
	for slot=1,3 do
		loadout[1],loadout[2],loadout[3],loadout[4] = C_PetJournal.GetPetLoadOutInfo(slot)
		for i=1,4 do
			if (loadin[slot][i]==loadout[i]) or (loadin[slot][i]==rematch.emptyPetID and not loadout[i]) then
				loadin[slot][i] = nil
			elseif loadin[slot][i] then
				debugp("pet slot",slot,"index",i,"(",unpack(loadin[slot]),") didn't load.")
			end
		end
	end

	-- if anything left in loadin, return false; everything didn't load
	for i=1,3 do
		for j=1,4 do
			if loadin[i][j] then
				return false
			end
		end
	end

	-- if we made it this far, return true; everything loaded
	return true
end

function rematch:LoadTeam(teamName)

	local team = saved[teamName]
	if not team then return end

	rematch.loadInProgress = true

	loadingTeamName = teamName

	rematch:ValidateTeam(team)

	rematch.header.text:SetText(L["Loading..."])
	rematch:DesaturatePets(true)

	-- wipe loadout; slots/abilities to ignore will be nil
	for i=1,3 do
		wipe(loadin[i])
	end

	local levelingAllocated

	-- fill loadin from the team
	for i=1,3 do
		local petID = team[i][1]
		if petID==0 and not levelingAllocated then
			local levelingPetID = rematch:GetCurrentLevelingPet()
			loadin[i][1] = levelingPetID or rematch.emptyPetID
			levelingAllocated = true
		elseif not petID or petID==rematch.emptyPetID then
			loadin[i][1] = rematch.emptyPetID
		elseif type(petID)=="string" then
			if C_PetJournal.PetIsSummonable(petID) and not C_PetJournal.PetIsRevoked(petID) then
				loadin[i][1] = petID
				for j=1,3 do
					local abilityID = team[i][j+1]
					if abilityID and abilityID~=0 then
						loadin[i][j+1] = abilityID
					end
				end
			end
		elseif settings.EmptyMissing then
			loadin[i][1] = rematch.emptyPetID
		end
	end

	if rematch:LoadLoadIn() then
		debugp(teamName,"loaded successfully first try!")
		rematch:LoadingDone()
	else
		debugp("** something didn't load from",teamName,"**")
		loadtimeout = 0
		rematch:StartTimer("ReloadLoadIn",0.2,rematch.ReloadLoadIn)
	end

end

function rematch:ReloadLoadIn()
	if rematch:LoadLoadIn() then
		debugp(loadingTeamName,"loaded successfully in",loadtimeout,"tries!")
		rematch:LoadingDone()
	elseif loadtimeout<20 then
		loadtimeout = loadtimeout + 1
		debugp("retrying again. loadtimeout=",loadtimeout)
		rematch:StartTimer("ReloadLoadIn",0.25,rematch.ReloadLoadIn)
	else
		debugp("giving up on",loadingTeamName,"!")
		rematch:LoadingDone(true)
	end
end

function rematch:DesaturatePets(value)
	for i=1,3 do
		rematch.current.pets[i].icon:SetDesaturated(value)
		for j=1,3 do
			rematch.current.pets[i].abilities[j].icon:SetDesaturated(value)
		end
	end
	rematch.current.back:SetDesaturated(value)
end

function rematch:LoadingDone(unsuccessful)

	if unsuccessful then
		-- in the rare event a load unsuccessful (lag?), nil loaded team
		settings.loadedTeamName = nil
		settings.loadedNpcID = nil
		rematch.lastOffered = nil
	else
		-- set loaded team
		settings.loadedTeamName = loadingTeamName
		settings.loadedNpcID = saved[loadingTeamName][4]
		if rematch:IsVisible() then
			for i=1,3 do
				rematch.current.pets[i].shine:SetCooldown(0,1)
			end
		end
	end

	rematch:DesaturatePets(false)
	rematch.loadInProgress = nil

	if PetJournal then
		PetJournal_UpdatePetLoadOut()
	end

	if rematch:IsVisible() then
		rematch:UpdateWindow()
	else
		-- if window isn't up, see if any pets are hurt to show window
		for i=1,3 do
			local petID = C_PetJournal.GetPetLoadOutInfo(i)
			if team[i][1] and team[i][1]~=rematch.emptyPetID and team[i][1]~=petID then
				rematch:Toggle()
				return
			elseif petID then
				local health,maxHealth = C_PetJournal.GetPetStats(petID)
				if health<1 then
					rematch:Toggle()
					return
				elseif health<(maxHealth*.9) and settings.ShowOnInjured then
					rematch:Toggle()
					return
				end
			end
		end
	end

end
